# encoding: utf-8

require 'json'
require 'logger'
require 'logstash/codecs/base'
require 'logstash/namespace'
require 'logstash/serializer'

# Read serialized Thrift Snowplow enriched-event records as Logstash events
#
# This plugin is used to serialize Logstash events as
# Snowplow enriched-event objects.
#
# ==== Encoding
#
# This codec is for serializing individual Logstash events as Snowplow enriched-event.
#
#
# ==== Decoding
#
# This codec is for deserializing individual Snowplow enriched-event serialized objects as Logstash events.
#
#
# ==== Usage
# Example usage with kafka input.
#
# [source,ruby]
# ----------------------------------
# input {
#   kafka {
#     codec => snowplow {
#     }
#   }
# }
# filter {
#   ...
# }
# output {
#   ...
# }
# ----------------------------------

class LogStash::Codecs::Snowplow < LogStash::Codecs::Base
  config_name 'snowplow'
  config :json_fields, validate: :array, default: %w[contexts derived_contexts se_property unstruct_event], required: true

  public
  def register
    @logger.info("Initializing logstash snowplow codec: json fields: #{@json_fields}!")
  end

  public
  def decode(data)
    event = Serializer.deserialize(data, @json_fields)
    yield LogStash::Event.new(event)
  rescue
    @logger.error("Fail to decode: #{data}")
  end
end
